/* $Id: fsconsole.c,v 1.1 2008/07/09 10:30:31 stacktic Exp $ */

/*
 * Copyright (c) 2008 Arnaud Ysmal.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/syslimits.h>

#include <assert.h>
#include <dirent.h>
#include <err.h>
#include <pwd.h>
#include <grp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <time.h>
#include <tzfile.h>
#include <unistd.h>

#include "ukfs.h"
#include "extern.h"
#include "fsu_mount.h"
#include "fsu_utils.h"
#include "fsconsole.h"

static int fsconsole_loop(void);
static int fsconsole_readline(char *, int);
static char *fsconsole_str2arg(char *, int, int *, char **);

extern char cwd[], old_wd[];
extern command_t commands[];

char *fstype, *fsdevice;
struct ukfs *ukfs;

int
fsu_sh(char *fst, char *fsd)
{
	int rv;

	fstype = fst;
	fsdevice = fsd;

	ukfs = fsu_mount(fst, fsd, 0);
	if (ukfs == NULL)
		err(EXIT_FAILURE, "%s", fsdevice);

	cwd[0] = old_wd[0] = '/';
	cwd[1] = old_wd[1] = '\0';
	
	rv = fsconsole_loop();
	/* NOTREACHED */

	assert(0);
	exit(EXIT_FAILURE); /* make GCC happy */
}

static int
fsconsole_loop(void)
{
	char *argv[MAX_OPTIONS+1], buffer[MAX_LINE_SIZE+1], *command;
	int argc = 0;
	command_t *com;
	int len;

	for (;;) {
		optind = optreset = 1;
		printf("%s(%s):%s # ", fsdevice, fstype, cwd);
		
		len = fsconsole_readline(buffer, MAX_LINE_SIZE);
		if (len <= 0)
			continue;
		
		command = fsconsole_str2arg(buffer, MAX_LINE_SIZE, &argc, argv);
		
		for (com = commands; com->c_name != NULL; com++)
			if (strcmp(command, com->c_name) == 0) {
				com->c_fun(ukfs, argc, argv);
				break;
			}

		if (com->c_name == NULL)
			fprintf(stderr, "%s: command not found\n", command);
	}
	/* NOTREACHED */
	assert(0);
}

/* read a line from stdin */
static int
fsconsole_readline(char *buf, int bufsize)
{
	struct termios tm;
	int i, rv;
	tcflag_t old_lflag;
	cc_t old_verase;

	fflush(stdout);
	
	rv = tcgetattr(STDIN_FILENO, &tm);
	if (rv != 0) {
		warn("tcgetattr"); 
		return -1;
	}
	

	old_lflag = tm.c_lflag;
	old_verase = tm.c_cc[VERASE];

	tm.c_cc[VERASE] = '\b';
	tm.c_lflag |= ICANON | ECHOE;

	rv = tcsetattr(STDIN_FILENO, TCSANOW, &tm); 
	if (rv != 0) {
		warn("tcsetattr");
		return -1;
	}
	
	i = read(STDIN_FILENO, buf, bufsize) - 1;
	buf[i] = '\0';

	tm.c_cc[VERASE] = old_verase;
	tm.c_lflag = old_lflag;	

	rv = tcsetattr(STDIN_FILENO, TCSANOW, &tm);
	if (rv != 0) {
		warn("tcsetattr");
		return -1;
	}
	
	return i;
}

/* get argc and argv from a char * */
static char
*fsconsole_str2arg(char *str, int bufsize, int *argc, char **argv)
{

	argv[0] = strtok(str, " ");
	*argc = 1;
	while (*argc < MAX_OPTIONS && argv[*argc - 1] != NULL)
		argv[(*argc)++] = strtok(NULL, " ");
	
	--(*argc);
	argv[*argc] = NULL;

	return argv[0];
}
