/* $Id: main.c,v 1.1 2008/07/09 10:33:37 stacktic Exp $ */

/*
 * Copyright (c) 2008 Arnaud Ysmal.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <sys/mount.h>
#include <sys/param.h>
#include <sys/vnode.h>

#include <err.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "ukfs.h"

#include "fsu_mount.h"
#include "fsu_utils.h"

#include "fsu_ls.h"

static void usage(void);

int
main(int argc, char *argv[])
{
	struct ukfs *fs;
	int cur_arg, flags = 0, rv;
	
	setprogname(argv[0]);

	flags = fsu_ls_parse_arg(&argc, &argv);
	if (flags == -1 || argc < 2)
		usage();

	fs = fsu_mount(argv[0], argv[1], MNT_RDONLY);
	if (fs == NULL)
		err(EXIT_FAILURE, "%s", argv[1]);


	if (argc == 2) {
		rv = fsu_ls(fs, "/", flags);
		goto out;
	}

	for (rv = 0, cur_arg = 2; cur_arg < argc; ++cur_arg) {
		rv |= fsu_ls(fs, argv[cur_arg], flags);
		if (cur_arg != argc - 1)
			printf("\n");
	}

out:
	ukfs_release(fs, 1);
	return rv;
}

static void
usage(void)
{

	fprintf(stderr, 
		"usage: %s [-AacdFlnpRrSTtu] fstype fsdevice [dirname...]\n", 
		getprogname());
	exit(EXIT_FAILURE);
}
