/* $Id */

/*
 * Copyright (c) 2008 Arnaud Ysmal.  All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <dirent.h>
#include <err.h>
#include <stdlib.h>
#include <string.h>

#include "ukfs.h"

#include "fsu_dirent.h"

FSU_DIR
*fsu_opendir(struct ukfs *fs, char *path)
{
	FSU_DIR *dir;

	dir = malloc(sizeof(FSU_DIR));
	if (dir == NULL) {
		warn("malloc");
		return NULL;
	}
	dir->ukfs = fs;
	dir->path = strdup(path);
	dir->offset = dir->size = 0;
	return dir;
}

void
fsu_closedir(FSU_DIR *dir)
{

	if (dir == NULL)
		return;
	free(dir->path);
	free(dir);
}

struct dirent
*fsu_readdir(FSU_DIR *dir)
{
	struct dirent *dent;

	if (dir->size <= 0) {
		dir->size = ukfs_getdents(dir->ukfs, dir->path, &dir->offset,
					  dir->buffer, FSU_DIR_BUF_SIZE);
		if (dir->size == -1) {
			warn("%s", dir->path);
			return NULL;
		} else if (dir->size == 0)
			return NULL;
		dir->dent = (struct dirent *)dir->buffer;
	}

	dent = dir->dent;
	
	dir->size -= _DIRENT_SIZE(dir->dent);
	dir->dent = _DIRENT_NEXT(dir->dent);

	return dent;
}

void
fsu_rewinddir(FSU_DIR *dir)
{
	dir->offset = 0;
	dir->size = 0;
}
